import torch
import argparse
import os

from load_model import load_model
from transformers import GPT2TokenizerFast, GPT2LMHeadModel
import torch.nn.functional as F
import sampling
import numpy as np

def create_directory_if_not_exists(directory_path):
    if not os.path.exists(directory_path):
        os.makedirs(directory_path)
        print(f"Directory '{directory_path}' created.")
    else:
        print(f"Directory '{directory_path}' already exists.")

def main():
    parser = argparse.ArgumentParser(description="Generate some samples")
    parser.add_argument("--model_path", default="exp_local/openwebtext/x", type=str)
    parser.add_argument("--dataset", default="wikitext103", type=str)
    parser.add_argument("--batch_size", type=int, default=256)
    parser.add_argument("--length", type=int, default=128)
    parser.add_argument("--no_batches", type=int, default=15)
    parser.add_argument("--steps", type=int, default=1024)
    args = parser.parse_args()

    
    device = torch.device('cuda')
    model, graph, noise = load_model(args.model_path, device)
    tokenizer = GPT2TokenizerFast.from_pretrained('gpt2')

    sampling_fn = sampling.get_pc_sampler(
        graph, noise, (args.batch_size, args.length), 'analytic', args.steps, device=device
    )



    with torch.no_grad():
        create_directory_if_not_exists(args.model_path+'_eval/')
        eval_model = GPT2LMHeadModel.from_pretrained("gpt2-large").to(device).eval()
        total_perplexity = []
        for i in range(args.no_batches):
            print(i)
            s = sampling_fn(model)
            _, logits = eval_model(s, labels=s)[:2]
            perplexity = F.cross_entropy(logits[:, :-1].reshape(-1, logits.size(-1)), s[..., 1:].reshape(-1), reduction='none')
            perplexity = perplexity.reshape(-1, args.length-1).mean(-1).exp().to(torch.float64).cpu().numpy().tolist()
            total_perplexity += perplexity    
            print(np.array(total_perplexity).mean())
            np.save(args.model_path+'_eval/'+'GenPerplexity_'+'totalpoints_'+str(args.no_batches*args.batch_size)+'_length_'+str(args.length)+'.npy', np.array(total_perplexity))

if __name__=="__main__":
    main()